/* This file contains both declaration and definition of the TestFakeSelection,
 * a GObject that pretends to implement the AtkSelectionIface interface (it 
 * registers appropriate interface), but provides no implementation for any of the
 * methods of this interface (NULL-filled vftbl).
 */

#ifndef TESTFAKESELECTION_H_
#define TESTFAKESELECTION_H_

#include <glib-object.h>
#include <atk/atk.h> 

#define TEST_TYPE_FAKE_SELECTION             (test_fake_selection_get_type ())
#define TEST_FAKE_SELECTION(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), TEST_TYPE_FAKE_SELECTION, TestFakeSelection))
#define TEST_FAKE_SELECTION_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST ((vtable), TEST_TYPE_FAKE_SELECTION, TestFakeSelectionClass))
#define TEST_IS_FAKE_SELECTION(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TEST_TYPE_FAKE_SELECTION))
#define TEST_IS_FAKE_SELECTION_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE ((vtable), TEST_TYPE_FAKE_SELECTION))
#define TEST_FAKE_SELECTION_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_CLASS ((inst), TEST_TYPE_FAKE_SELECTION, TestFakeSelectionClass))

typedef struct _TestFakeSelection TestFakeSelection;
typedef struct _TestFakeSelectionClass TestFakeSelectionClass;

struct _TestFakeSelection 
{
    GObject parent;
        
    gboolean disposed;
};

struct _TestFakeSelectionClass 
{
    GObjectClass parent;
};

GType 
test_fake_selection_get_type (void);

/******************************************************************/
/*                    Implementation                              */
/******************************************************************/
static GObjectClass *parent_class_fake_selection = NULL;

/******************************************************************/
static void
fake_selection_interface_init (gpointer g_iface, gpointer iface_data)
{
    AtkSelectionIface *klass = (AtkSelectionIface *)g_iface;
    
    /* set up overrides here */
    klass->add_selection = 
        (gboolean (*) (AtkSelection *selection, gint i)) NULL;
    
    klass->clear_selection = 
        (gboolean (*) (AtkSelection *selection)) NULL;
    
    klass->ref_selection = 
        (AtkObject* (*) (AtkSelection *selection, gint i)) NULL;
    
    klass->get_selection_count = 
        (gint (*) (AtkSelection *selection)) NULL;
    
    klass->is_child_selected = 
        (gboolean (*) (AtkSelection *selection, gint i)) NULL;
    
    klass->remove_selection = 
        (gboolean (*) (AtkSelection *selection, gint i)) NULL;
    
    klass->select_all_selection = 
        (gboolean (*) (AtkSelection *selection)) NULL;
}

static void
fake_selection_instance_init (GTypeInstance *instance, gpointer g_class)
{
    TestFakeSelection *self = (TestFakeSelection *)instance;
    
    self->disposed = FALSE;
}

static void
test_fake_selection_dispose (GObject *obj)
{
    TestFakeSelection *self = (TestFakeSelection *)obj;

    if (self->disposed) 
    {
        return;
    }
    
    /* Make sure dispose does not run twice. */
    self->disposed = TRUE;

    /* Chain up to the parent class */
    G_OBJECT_CLASS (parent_class_fake_selection)->dispose (obj);
}

static void
test_fake_selection_finalize (GObject *obj)
{
    /*TestFakeSelection *self = (TestFakeSelection *)obj;
    if (self)
    {
    }*/
    
    /* Chain up to the parent class */
    G_OBJECT_CLASS (parent_class_fake_selection)->finalize (obj);
}

static void
test_fake_selection_class_init (gpointer g_class, gpointer g_class_data)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
    TestFakeSelectionClass *klass = TEST_FAKE_SELECTION_CLASS (g_class);

    gobject_class->dispose = test_fake_selection_dispose;
    gobject_class->finalize = test_fake_selection_finalize;

    parent_class_fake_selection = g_type_class_peek_parent (klass);
}

GType 
test_fake_selection_get_type (void)
{
    static GType type = 0;
    if (type == 0) 
    {
        static const GTypeInfo info = 
        {
            sizeof (TestFakeSelectionClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            test_fake_selection_class_init, /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof (TestFakeSelection),
            0,      /* n_preallocs */
            fake_selection_instance_init    /* instance_init */
        };
                
        static const GInterfaceInfo iface_info = 
        {
            (GInterfaceInitFunc) fake_selection_interface_init,    /* interface_init */
            NULL,                                       /* interface_finalize */
            NULL                                        /* interface_data */
        };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "TestFakeSelectionType",
                                       &info, 0);
        g_type_add_interface_static (type,
                                     ATK_TYPE_SELECTION,
                                     &iface_info);
    }
    return type;
}

#endif /*TESTFAKESELECTION_H_*/

