/* This file contains both declaration and definition of the TestFakeTable,
 * a GObject that pretends to implement the AtkTableIface interface (it 
 * registers appropriate interface), but provides no implementation for any of the
 * methods of this interface (NULL-filled vftbl).
 */

#ifndef TESTFAKETABLE_H_
#define TESTFAKETABLE_H_

#include <glib-object.h>
#include <atk/atk.h> 
    
#include <AtkTable/test-simple-table.h>

#define TEST_TYPE_FAKE_TABLE             (test_fake_table_get_type ())
#define TEST_FAKE_TABLE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), TEST_TYPE_FAKE_TABLE, TestFakeTable))
#define TEST_FAKE_TABLE_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST ((vtable), TEST_TYPE_FAKE_TABLE, TestFakeTableClass))
#define TEST_IS_FAKE_TABLE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TEST_TYPE_FAKE_TABLE))
#define TEST_IS_FAKE_TABLE_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE ((vtable), TEST_TYPE_FAKE_TABLE))
#define TEST_FAKE_TABLE_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_CLASS ((inst), TEST_TYPE_FAKE_TABLE, TestFakeTableClass))

typedef struct _TestFakeTable TestFakeTable;
typedef struct _TestFakeTableClass TestFakeTableClass;

struct _TestFakeTable 
{
    GObject parent;
        
    gboolean disposed;
    
    TestSimpleChild* child[NCHILDREN];
    TestSimpleChild* not_a_child;
    TestSimpleHeaderStruct row[NROWS];
    TestSimpleHeaderStruct col[NCOLS];
    guint nrows;
    guint ncols;
    AtkObject* caption;
    AtkObject* summary;
};

struct _TestFakeTableClass 
{
    GObjectClass parent;
};

GType 
test_fake_table_get_type (void);

///////////////////////////////////////////////////////////////////////////
// Helper functions and data
///////////////////////////////////////////////////////////////////////////
void
test_fake_table_select_rows (TestFakeTable* table, gboolean sel_scheme[])
{
    // the function does nothing  
}

void
test_fake_table_select_columns (TestFakeTable* table, gboolean sel_scheme[])
{
    // the function does nothing
}

///////////////////////////////////////////////////////////////////////////
// Implementation
///////////////////////////////////////////////////////////////////////////
static GObjectClass *parent_class_fake_table = NULL;

/******************************************************************/
static void
fake_table_interface_init (gpointer g_iface, gpointer iface_data)
{
    AtkTableIface *klass = (AtkTableIface *)g_iface;
    
    /* set up overrides here */
    klass-> ref_at =
        (AtkObject* (*) (AtkTable *table, gint row, gint column)) NULL;
    klass-> get_index_at =
        (gint (*) (AtkTable *table, gint row, gint column)) NULL;
    klass-> get_column_at_index =
        (gint (*) (AtkTable *table, gint index_)) NULL;
    klass-> get_row_at_index =
        (gint (*) (AtkTable *table, gint index_)) NULL;
    klass-> get_n_columns =
        (gint (*) (AtkTable *table)) NULL;
    klass-> get_n_rows =
        (gint (*) (AtkTable *table)) NULL;
    klass-> get_column_extent_at =
        (gint (*) (AtkTable *table, gint row, gint column)) NULL;
    klass-> get_row_extent_at =
        (gint (*) (AtkTable *table, gint row, gint column)) NULL;
    klass-> get_caption =
        (AtkObject* (*) (AtkTable *table)) NULL;
    klass-> get_column_description =
        (const gchar* (*) (AtkTable *table, gint column)) NULL;
    klass-> get_column_header =
        (AtkObject* (*) (AtkTable *table, gint column)) NULL;
    klass-> get_row_description =
        (const gchar* (*) (AtkTable *table, gint row)) NULL;
    klass-> get_row_header =
        (AtkObject* (*) (AtkTable *table, gint row)) NULL;
    klass-> get_summary =
        (AtkObject* (*) (AtkTable *table)) NULL;
    klass-> set_caption =
        (void (*) (AtkTable *table, AtkObject *caption)) NULL;
    klass-> set_column_description =
        (void (*) (AtkTable *table, gint column, const gchar *description)) NULL;
    klass-> set_column_header =
        (void (*) (AtkTable *table, gint column, AtkObject *header)) NULL;
    klass-> set_row_description =
        (void (*) (AtkTable *table, gint row, const gchar *description)) NULL;
    klass-> set_row_header =
        (void (*) (AtkTable *table, gint row, AtkObject *header)) NULL;
    klass-> set_summary =
        (void (*) (AtkTable *table, AtkObject *accessible)) NULL;
    klass-> get_selected_columns =
        (gint (*) (AtkTable *table, gint **selected)) NULL;
    klass-> get_selected_rows =
        (gint (*) (AtkTable *table, gint **selected)) NULL;
    klass-> is_column_selected =
        (gboolean (*) (AtkTable *table, gint column)) NULL;
    klass-> is_row_selected =
        (gboolean (*) (AtkTable *table, gint row)) NULL;
    klass-> is_selected =
        (gboolean (*) (AtkTable *table, gint row, gint column)) NULL;
    klass-> add_row_selection =
        (gboolean (*) (AtkTable *table, gint row)) NULL;
    klass-> remove_row_selection =
        (gboolean (*) (AtkTable *table, gint row)) NULL;
    klass-> add_column_selection =
        (gboolean (*) (AtkTable *table, gint column)) NULL;
    klass-> remove_column_selection =
        (gboolean (*) (AtkTable *table, gint column)) NULL;
}

static void
fake_table_instance_init (GTypeInstance *instance, gpointer g_class)
{
    TestFakeTable *self = (TestFakeTable *)instance;
    
    self->disposed = FALSE;
}

static void
test_fake_table_dispose (GObject *obj)
{
    TestFakeTable *self = (TestFakeTable *)obj;

    if (self->disposed) 
    {
        return;
    }
    
    /* Make sure dispose does not run twice. */
    self->disposed = TRUE;

    /* Chain up to the parent class */
    G_OBJECT_CLASS (parent_class_fake_table)->dispose (obj);
}

static void
test_fake_table_finalize (GObject *obj)
{
    /* Chain up to the parent class */
    G_OBJECT_CLASS (parent_class_fake_table)->finalize (obj);
}

static void
test_fake_table_class_init (gpointer g_class, gpointer g_class_data)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
    TestFakeTableClass *klass = TEST_FAKE_TABLE_CLASS (g_class);

    gobject_class->dispose = test_fake_table_dispose;
    gobject_class->finalize = test_fake_table_finalize;

    parent_class_fake_table = g_type_class_peek_parent (klass);
}

GType 
test_fake_table_get_type (void)
{
    static GType type = 0;
    if (type == 0) 
    {
        static const GTypeInfo info = 
        {
            sizeof (TestFakeTableClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            test_fake_table_class_init, /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof (TestFakeTable),
            0,      /* n_preallocs */
            fake_table_instance_init    /* instance_init */
        };
                
        static const GInterfaceInfo iface_info = 
        {
            (GInterfaceInitFunc) fake_table_interface_init,    /* interface_init */
            NULL,                                       /* interface_finalize */
            NULL                                        /* interface_data */
        };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "TestFakeTableType",
                                       &info, 0);
        g_type_add_interface_static (type,
                                     ATK_TYPE_TABLE,
                                     &iface_info);
    }
    return type;
}

#endif /*TESTFAKETABLE_H_*/

