#ifndef MY_ATK_UTILS_H
#define MY_ATK_UTILS_H

#include <AtkUtils/user_marshal.h>
/*
 * Our realization of mechanism for adding/removing listeners 
 */

guint global_event_signal_id = 0;
guint key_event_signal_id = 0;

void class_callback_global_event(AtkUtil* object)
{
};
gint class_callback_key_event(AtkUtil* object)
{
    return 0;
};
AtkUtil* atkUtil = NULL;//need for emission of a signal
//global_event
//incorrect types for listener_id(guint instead if gulong), but it's probably used in atk
guint my_atk_add_global_event_listener(GSignalEmissionHook listener, const gchar* event_type)
{
    return (guint)g_signal_add_emission_hook(global_event_signal_id,
                                g_quark_from_string(event_type),
                                listener,
                                NULL,
                                NULL);
}
void my_atk_remove_global_event_listener(guint listener_id)
{
    g_signal_remove_emission_hook(global_event_signal_id, (gulong)listener_id);
}
//key_event
AtkKeyEventStruct* my_atk_key_event_struct_copy(AtkKeyEventStruct* src)
{
    if(src == NULL) return NULL;
    AtkKeyEventStruct* result = g_new(AtkKeyEventStruct, 1);
    *result = *src;
    result->string = g_strdup(result->string);
    return result;
}
void my_atk_key_event_struct_free(AtkKeyEventStruct* src)
{
    if(src == NULL) return;
    g_free(src->string);
    g_free(src);
}

GType my_atk_key_event_struct_get_type()
{
    static GType type = 0;
    if(type == 0)
    {
        type = g_boxed_type_register_static("AtkKeyEventStruct",
                                             (GBoxedCopyFunc)my_atk_key_event_struct_copy,
                                             (GBoxedFreeFunc)my_atk_key_event_struct_free);
    }
    return type;
}
#define MY_TYPE_ATK_KEY_EVENT_STRUCT my_atk_key_event_struct_get_type()
typedef struct
{
    AtkKeySnoopFunc listener;
    gpointer func_data;
}CallbackDataEvent;
//A bridge to eliminate the first parameter of the callback function for the signal
gint key_event_listener_bridge(AtkUtil* atkUtil, AtkKeyEventStruct* event, CallbackDataEvent* data)
{
    return data->listener(event, data->func_data);
}
//same problem as for my_atk_add_global_event_listener
guint my_atk_add_key_event_listener(AtkKeySnoopFunc listener, gpointer data)
{
    CallbackDataEvent* callback_data = g_new(CallbackDataEvent, 1);
    callback_data->listener = listener;
    callback_data->func_data = data;
    
    return (guint)g_signal_connect_data(atkUtil,
                                "key-event",
                                (GCallback)key_event_listener_bridge,
                                callback_data,
                                (GClosureNotify)g_free,
                                0);
}
void my_atk_remove_key_event_listener(guint listener_id)
{
    g_signal_handler_disconnect(atkUtil, (gulong)listener_id);
}

// load my realization of add/remove listener mechanism to the class
// (and therefore, to the system)
void my_atk_util_class_init()
{
    if (global_event_signal_id != 0) return; //already initialized
    
    global_event_signal_id = g_signal_newv("global-event",
                ATK_TYPE_UTIL,
                G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                g_cclosure_new((GCallback)class_callback_global_event, NULL, NULL),
                NULL,
                NULL,
                g_cclosure_marshal_VOID__VOID,
                G_TYPE_NONE,
                0,NULL);
    
    GType param_types[] = {MY_TYPE_ATK_KEY_EVENT_STRUCT/* | G_SIGNAL_TYPE_STATIC_SCOPE*/}; 
    key_event_signal_id = g_signal_newv("key-event",
                ATK_TYPE_UTIL,
                G_SIGNAL_RUN_LAST,
                g_cclosure_new((GCallback)class_callback_key_event, NULL, NULL),
                NULL,
                NULL,
                g_cclosure_user_marshal_INT__BOXED,
                G_TYPE_INT,
                1,
                param_types);
    
    atkUtil = g_object_new(ATK_TYPE_UTIL, NULL);
    
    AtkUtilClass* atkUtilClass = ATK_UTIL_CLASS(g_type_class_ref(ATK_TYPE_UTIL));

    atkUtilClass->add_global_event_listener = my_atk_add_global_event_listener;
    atkUtilClass->remove_global_event_listener = my_atk_remove_global_event_listener;
    atkUtilClass->add_key_event_listener = my_atk_add_key_event_listener;
    atkUtilClass->remove_key_event_listener = my_atk_remove_key_event_listener;

    g_type_class_unref(atkUtilClass);
}
void generate_global_event(const gchar* event_type)
{
    g_signal_emit(atkUtil, global_event_signal_id, g_quark_from_string(event_type));
}
gint generate_key_event(AtkKeyEventStruct* event)
{
    gint result = 0;
    g_signal_emit(atkUtil, key_event_signal_id, 0, event, &result);
    return result;
}

#endif /*!MY_ATK_UTILS_H*/
