#ifndef MY_ATK_VALUE_H
#define MY_ATK_VALUE_H
/*
 * MyAtkValue: derives AtkObject
 * and implements AtkValue
 */
#include <atk/atk.h>
#include <glib-object.h>
#include <limits.h>

#define MY_TYPE_ATK_VALUE             (my_atk_value_get_type ())
#define MY_ATK_VALUE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), MY_TYPE_ATK_VALUE, MyAtkValue))
#define MY_ATK_VALUE_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST ((vtable), MY_TYPE_ATK_VALUE, MyAtkValueClass))
#define MY_IS_ATK_VALUE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MY_TYPE_ATK_VALUE))
#define MY_IS_ATK_VALUE_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE ((vtable), MY_TYPE_ATK_VALUE))
#define MY_ATK_VALUE_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_CLASS ((inst), MY_TYPE_ATK_VALUE, MyAtkValueClass))

typedef struct _MyAtkValue MyAtkValue;
typedef struct _MyAtkValueClass MyAtkValueClass;

struct _MyAtkValue
{
    AtkObject parent;
    
    gint minimum, maximum, current;
    gboolean readonly;
};

struct _MyAtkValueClass
{
    AtkObjectClass parent;
};

MyAtkValue* my_atk_value_new(gint minimum, gint maximium, gint current);
GType my_atk_value_get_type();
//*************************implementation***********************
//implementation of virtual functions
//******************get_current_value*******************
static void my_atk_value_get_current_value(AtkValue *obj, GValue *value)
{
    g_return_if_fail(MY_IS_ATK_VALUE(obj));
    MyAtkValue* self = (MyAtkValue*)obj;
    
    g_value_init(value, G_TYPE_INT);
    g_value_set_int(value, self->current);
}
//******************get_maximum_value*******************
static void my_atk_value_get_maximum_value(AtkValue *obj, GValue *value)
{
    g_return_if_fail(MY_IS_ATK_VALUE(obj));
    MyAtkValue* self = (MyAtkValue*)obj;
    
    g_value_init(value, G_TYPE_INT);
    g_value_set_int(value, self->maximum);
}
//******************get_minimum_value*******************
static void my_atk_value_get_minimum_value(AtkValue *obj, GValue *value)
{
    g_return_if_fail(MY_IS_ATK_VALUE(obj));
    MyAtkValue* self = (MyAtkValue*)obj;
    
    g_value_init(value, G_TYPE_INT);
    g_value_set_int(value, self->minimum);
}
//******************set_current_value*******************
static gboolean my_atk_value_set_current_value(AtkValue *obj, const GValue *value)
{
    g_return_val_if_fail(MY_IS_ATK_VALUE(obj), FALSE);
    MyAtkValue* self = (MyAtkValue*)obj;
    
    if(self->readonly) return FALSE;
    
    gint new_value = g_value_get_int(value);
    if(new_value < self->minimum || new_value > self->maximum) return FALSE;
    
    self->current = new_value;
    return TRUE;
}

//others
MyAtkValue* my_atk_value_new(gint minimum, gint maximum, gint current)
{
    MyAtkValue* result = g_object_new(MY_TYPE_ATK_VALUE, NULL);
    if(result == NULL) return NULL;
    result->minimum = minimum;
    result->maximum = maximum;
    result->current = current;
    
    return result;
}
static void my_atk_value_instance_init(GTypeInstance *obj, gpointer g_class)
{
    MyAtkValue *self = (MyAtkValue*)obj;
    
    self->minimum = 0;
    self->maximum = 0;
    self->current = 0;
    
    self->readonly = FALSE;
}
static void my_atk_value_interface_init(gpointer g_iface, gpointer iface_data)
{
    AtkValueIface *klass = (AtkValueIface*)g_iface;
    
    klass->get_current_value = my_atk_value_get_current_value;
    klass->get_minimum_value = my_atk_value_get_minimum_value;
    klass->get_maximum_value = my_atk_value_get_maximum_value;
    
    klass->set_current_value = my_atk_value_set_current_value;
}

GType my_atk_value_get_type()
{
    static GType type = 0;
    if(type == 0)
    {
        static const GTypeInfo typeInfo = 
        {
            sizeof(MyAtkValueClass),
            NULL, //base_init
            NULL, //base_finalize
            NULL, //class_init
            NULL, //class_finalize
            NULL, //class_data
            sizeof(MyAtkValue),
            0, //n_preallocs
            my_atk_value_instance_init //instance_init
        };

        static const GInterfaceInfo iface_info = 
        {
            my_atk_value_interface_init,        /* interface_init*/
            NULL,                               /* interface_finalize*/
            NULL                                /* interface_data */
        };
        type = g_type_register_static(ATK_TYPE_OBJECT, "MyAtkValue", &typeInfo, 0);
        g_type_add_interface_static(type,
            ATK_TYPE_VALUE,
            &iface_info);
    }
    return type;    
}
#endif /*MY_ATK_VALUE_H*/
