#ifndef USEFUL_FUNCTIONS_H
#define USEFUL_FUNCTIONS_H

/*
 * Functions and macros which is widely used in tests.
 */
 
//macro for create objects in startup section
#define OBJECT_NEW(obj, type, type_str) obj = g_object_new(type,NULL);\
    if(obj == NULL)\
    {\
        INIT_FAILED("Cannot create instance of type" type_str ".\n");\
    }
//macro for destroy object(preliminarily test it on NULL)
#define OBJECT_UNREF(obj) if(obj != NULL)\
    {\
        g_object_unref((gpointer)obj);\
    }
//for test signals
#define HANDLER_DISCONNECT(obj, h) if((h) != 0)\
	{\
		g_signal_handler_disconnect(obj, h);\
	}
//same as strcmp() == 0, but correct work with NULL-strings
gboolean my_strcmp(const gchar* str1, const gchar* str2)
{
    if(str1 == str2) return TRUE;
    if(str1 == NULL || str2 == NULL) return FALSE;
    
    return strcmp(str1,str2) == 0;
}
//same as strlen, but correct work with NULL-string and return gint instead of guint
gint my_strlen(const gchar* str)
{
    if(str == NULL)return 0;
    return (gint)strlen(str);
}
//same as strncmp() == 0, but correct work with NULL-strings
gboolean my_strncmp(const gchar* str1, const gchar* str2, gint n)
{
    if(n <= 0)return TRUE;
    if(str1 == str2)return TRUE;
    if(str1 == NULL || str2 == NULL)return FALSE;

    return strncmp(str1, str2, n) == 0;
}
 
#endif /*USEFUL_FUNCTIONS_H*/
