
#include <stdio.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include "spawn_test.h"

int init_psem (int sid)
{
    struct sembuf lock_it;
    lock_it.sem_num = 0;
    lock_it.sem_op = 1;
    lock_it.sem_flg = SEM_UNDO;
    return semop (sid, &lock_it, 1);
}

int get_psem (int create)
{
    key_t u_key = ftok ("/tmp", 7654321);
    int flags = (create ?  IPC_CREAT : 0) | 0666;
    int sem_id = semget (u_key, 1, flags);
    if (create)
    {
        init_psem (sem_id);
    }
    return sem_id;
}

int unlock_psem (int sid)
{
    struct sembuf lock_it;
    lock_it.sem_num = 0;
    lock_it.sem_op = -1;
    lock_it.sem_flg = IPC_NOWAIT;
    return semop (sid, &lock_it, 1);
}

int wait_psem(int sid)
{
    struct sembuf lock_it;
    lock_it.sem_num = 0;
    lock_it.sem_op = 0;
    lock_it.sem_flg = SEM_UNDO;
    return semop(sid, &lock_it, 1);
}

/* Return true if this socket is readable right now.*/
int is_readable (int fd)
 {
    fd_set read;
    struct timeval to;
    to.tv_sec=0;
    to.tv_usec=1000;
    FD_ZERO(&read);
    FD_SET(fd,&read);
    if (-1==select(fd+1,&read,NULL,NULL,&to))
        return 0;          /* timeout */
    return FD_ISSET(fd,&read)?1:0;
}

int main (int argc, char *argv[])
{
    int semaphore = get_psem (0);
    if (strcmp (argv[2], "async_with_pipes.05") == 0 || strcmp (argv[2], "async.07") == 0)
    {
         wait_psem (semaphore);
    }
    else
    init_psem (semaphore);
    FILE* fp;
    char *env = NULL;
    int fd = 0;
     
// file where arguments will be written
    if ((fd = open (argv[argc-1], O_CREAT|O_TRUNC|O_WRONLY|O_SYNC,S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
    {
        unlock_psem (semaphore);
        printf("cannot open file");
        return -1;
    }
    	
    fp = fdopen(fd,"w");


    // if env var was sent 
    env = getenv ("LSB_SPAWN_TEST");

    if (env != NULL)
    {
        fprintf (fp, env);
        fprintf (fp, "\n");

        goto cleanup;
    }

    // G_SPAWN_CHILD_INHERITS_STDIN flag checking
    if (strcmp (argv[2], "stdin") == 0 )
    {
        char buffer[100];
        // read from stdin 
        scanf ("%s", buffer);
        // write it to args file
        fprintf (fp, buffer);
        printf (STDOUT_TEXT);
        fprintf (stderr, STDERR_TEXT);
        goto cleanup;
    }
    if (strcmp (argv[2], "stdinisnull") == 0 )
    {
        int stdinno = fileno(stdin); 
        if(!is_readable(stdinno))
        {
            fprintf(fp, "stdin is not readable");
        }
        else
        {
           char buffer[100];
            // read from stdin 
            int readret= read (stdinno,buffer,1);
            if(readret==0)
            fprintf(fp, "stdinisnull");
        }
        goto cleanup;
    }

    if (strcmp (argv[2], "stdout") == 0 )
    {
        printf (STDOUT_TEXT);
        fprintf (stderr, STDERR_TEXT);
        goto cleanup;
    }

    //G_SPAWN_STDERR_TO_DEV_NULL
    if (strcmp (argv[2], "stderr") == 0 )
    {
        int stderrno = fileno(stderr);
        int isat = isatty(stderrno);
        if (isat == 0 || errno > 0) // the stderr is closed
        {
            printf("ok");
        }
        else
        {
            printf("no");
        }
        goto cleanup;
    }
    
    // open file descriptor
    // testing G_SPAWN_LEAVE_DESCRIPTORS_OPEN flag
    if (strcmp (argv[0], "spawn_test") == 0 && strcmp (argv[1], "1") != 0)
    {
        int fd = atoi (argv[1]);
        if (fd > 1)
        {
            int newfd = dup2 (fd, fd);
            fprintf (fp, "%d", newfd);
        }
        fprintf (fp, "\n");
        goto cleanup;
    }
    
    // standard behavior
    while (argc && *argv)
    {
        fprintf (fp, *argv);

        argc--, argv++;
    }
    fprintf (fp, "\n");


cleanup:
    fclose(fp);
    unlock_psem(semaphore);
    return 0;
}
