#ifndef GLIB_MESSAGES_H_
#define GLIB_MESSAGES_H_

#define GLIB_CRITICAL \
char* g_crit_err_str = NULL;\
_Bool err_expected = FALSE;\
guint t2c_log_handler_id;\
\
void t2c_gliberr_handler (const gchar   *log_domain,\
            GLogLevelFlags log_level,\
            const gchar   *message,\
            gpointer          unused_data);\
\
void t2c_gliberr_handler (const gchar   *log_domain,\
            GLogLevelFlags log_level,\
            const gchar   *message,\
            gpointer          unused_data)\
{\
    if(g_crit_err_str)\
        free(g_crit_err_str);\
    g_crit_err_str = g_strdup(message);\
}//End GLIB_CRITICAL


#define GLIB_CRITICAL_INIT() \
{\
    t2c_log_handler_id = g_log_set_handler ("GLib",  G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION | G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL, t2c_gliberr_handler, NULL);\
};

#define GLIB_CRITICAL_ERR_CLEAR() \
{\
    if(g_crit_err_str)\
    { \
        free(g_crit_err_str);\
        g_crit_err_str = NULL;\
    } \
};

#define GLIB_CRITICAL_CLEANUP() \
{\
    g_log_remove_handler ("GLib", t2c_log_handler_id);\
    if(g_crit_err_str)\
    { \
        free(g_crit_err_str);\
        g_crit_err_str = NULL;\
    } \
};\

#define SET_ERROR_EXPECTED(is_exp) err_expected = is_exp;

#define IS_ERROR_REPORTED (g_crit_err_str? TRUE:FALSE);

#define CHECK_GLIB_CRITICAL_ERROR() \
{\
    if(!err_expected && g_crit_err_str)\
    { \
        TRACE("Error reported in %s, line %d \n comment: [%s]\n", __FILE__, __LINE__,  g_crit_err_str); \
        free(g_crit_err_str);\
        g_crit_err_str = NULL;\
        ABORT_TEST_PURPOSE("Critical Glib error.");\
    } \
};

#define UNRES_GERROR(g_error)\
    {if (g_error != NULL) { \
        TRACE("Error Reported in %s, line %d \n comment: [%d: %s]\n", __FILE__, __LINE__,  g_error->code, g_error->message); \
        g_error_free (g_error);\
        g_error = NULL;\
        ABORT_TEST_PURPOSE("Test purpose error.");\
    }} \

#endif /*GLIB_MESSAGES_H_*/
