#ifndef GTCOMMON_H_
#define GTCOMMON_H_


#include <glib.h>
#include <glib-object.h>
#include <t2c.h>
#include "math.h"

//------------------------------------------------------------------------------------------------
// Utility structure for holding generic value

#define TVALUE_NONE 0
#define TVALUE_INT 1
#define TVALUE_UINT 2
#define TVALUE_LONG 3
#define TVALUE_FLOAT 4
#define TVALUE_DOUBLE 5
#define TVALUE_STRING 6


typedef struct _TValue{

	int valueType;

	gint intValue;
	guint uintValue;
	glong longValue;
	gfloat floatValue;
	gdouble doubleValue;

	gulong ulongValue;
	gchar charValue;
	guchar ucharValue;
	gboolean booleanValue;
	gint64 int64Value;
	guint64 uint64Value;

	guint flagsValue;
	gint enumValue;

	gchar* stringValue;

	gpointer pointerValue;
	gpointer boxedValue;
	GObject* objectValue;
	GParamSpec* paramspecValue;

} TValue;


void set_tvalue_int(TValue* tvalue, gint intValue);
void set_tvalue_uint(TValue* tvalue, guint uintValue);
void set_tvalue_long(TValue* tvalue, glong longValue);
void set_tvalue_float(TValue* tvalue, gfloat floatValue);
void set_tvalue_double(TValue* tvalue, gdouble doubleValue);
void set_tvalue_string(TValue* tvalue, gchar* stringValue);

void copy_tvalue(TValue* src, TValue* dest);
void copy_2_gvalue(TValue* src, GValue* dest);

void init_tvalue(TValue* tvalue);

//No type checking is performed during execution of this function
gboolean tvalue_holds_same_value(TValue* tvalue, GValue* exp_value);


//------------------------------------------------------------------------------------------------
// Utility functions declaration


gboolean gparamspec_equal(GParamSpec* pspec1, GParamSpec* pspec2);
void gparamspec_comparison_print(GParamSpec* paramSpec, GParamSpec* expParamSpec);

gboolean gvalue_equal(GValue* gvalue1, GValue* gvalue2);
void gvalue_free(gpointer data);

//------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------
// Global structures for diagnosing the validity of GObject operations (checklists)
//------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------
// Global structure holding values, to help diagnose the validity of operations with GTBase object

typedef struct _GObject_constructor_CT{

//Validation result variables, set in the overriden GObject constructor

	gint call_count;

	gboolean type_valid;
	gboolean n_construct_properties_valid;
	gboolean construct_properties_valid;

	gboolean cprops_initialized_w_def_values;

	gboolean properties_installed;

//Variables containing values describing initial conditions

	GType expected_type;
	gint initialValueCount;
	TValue *initialValues;

} GObject_constructor_CT;

GObject_constructor_CT base_constructor_ct;

//Required for TRACE macros
extern int bVerbose;


//-----------------------------------------------------------------------------------------------------
// Global structure holding values, to help diagnose the validity of operations with  GTRefObject object

typedef struct _GTRefObject_CT{

	gint finally_call_count;
	gboolean finally_pobject_valid;

    gint dispose_call_count;
    gboolean dispose_pobject_valid;

	GObject* finally_expectedObject;
    GObject* dispose_expectedObject;

} GTRefObject_CT;

GTRefObject_CT refobject_ct;

//-----------------------------------------------------------------------------------------------------
// Global structure holding values, to help diagnose the validity of operations with  GTPRBase, GTPRChild, GTPropBase and GTPropChild objects

//Values of the parameters passed to "([s|g]et_property)" are stored here
typedef struct _GTPRSGPropertyPValues{

	GObject *object;
	gboolean property_id_valid;
	GValue gvalue;
	GParamSpec *pspec;

} GTPRSGPropertyPValues;

//Values of the parameters passed to "dispatch_properties_changed" are stored here
typedef struct _GTDPCPropertyPValues{

	GObject *object;
	guint n_pspecs;
	GParamSpec **pspecs;

} GTDPCPropertyPValues;


typedef struct _GTPRObjects_CT{

	gboolean verify_prop_set;
	gboolean verify_prop_get;

	gboolean properties_installed;
	gboolean constructor_params_valid;
	gint property_value_set;
	gint property_value_got;
	gint gsetprop_call_count;
	gint ggetprop_call_count;

    //notify class closure parameters
    gint notify_cc_call_count;
    gboolean notify_cc_firstemstcall;
    gboolean notify_cc_pobject_valid;
    gboolean notify_cc_pspec_valid;
    gpointer notify_cc_exp_pobject;
    gpointer notify_cc_exp_pspec;

	GTPRSGPropertyPValues sgPValues;
	GTDPCPropertyPValues dpcPValues;

	GHashTable* propspec_map;
	GHashTable* propvalues_map;

} GTPRObjects_CT;

GTPRObjects_CT probjects_ct;

typedef struct _GTSigbjects_CT{

    gint dsfirst_cc_call_count;

    gint dslast_cc_call_count;

    gint osfirst_cc_call_count;
} GTSigbjects_CT;


GTSigbjects_CT sigobjects_ct;

#endif /*GTCOMMON_H_*/
