#include "gt-common.h"


void set_tvalue_int(TValue* tvalue, gint intValue)
{
	tvalue->valueType = TVALUE_INT;
	tvalue->intValue = intValue;
}

void set_tvalue_uint(TValue* tvalue, guint uintValue)
{
	tvalue->valueType = TVALUE_UINT;
	tvalue->uintValue = uintValue;
}

void set_tvalue_long(TValue* tvalue, glong longValue)
{
	tvalue->valueType = TVALUE_LONG;
	tvalue->longValue = longValue;
}

void set_tvalue_float(TValue* tvalue, gfloat floatValue)
{
	tvalue->valueType = TVALUE_FLOAT;
	tvalue->floatValue = floatValue;
}

void set_tvalue_double(TValue* tvalue, gdouble doubleValue)
{
	tvalue->valueType = TVALUE_DOUBLE;
	tvalue->doubleValue = doubleValue;
}

void set_tvalue_string(TValue* tvalue, gchar* stringValue)
{
	tvalue->valueType = TVALUE_STRING;
	tvalue->stringValue = stringValue;
}

void copy_tvalue(TValue* src, TValue* dest)
{
	dest->valueType = src->valueType;
	dest->intValue = src->intValue;
	dest->uintValue = src->uintValue;
	dest->longValue = src->longValue;
	dest->floatValue = src->floatValue;
	dest->doubleValue = src->doubleValue;
	dest->stringValue = src->stringValue;
}

void copy_2_gvalue(TValue* src, GValue* dest)
{


	switch(src->valueType){
		case TVALUE_INT:
			g_value_init(dest, G_TYPE_INT);
			g_value_set_int(dest, src->intValue);
			break;
		case TVALUE_UINT:
			g_value_set_uint(dest, src->uintValue);
			break;
		case TVALUE_LONG:
			g_value_set_long(dest, src->longValue);
			break;
		case TVALUE_FLOAT:
			g_value_init(dest, G_TYPE_FLOAT);
			g_value_set_float(dest, src->floatValue);
			break;
		case TVALUE_DOUBLE:
			g_value_init(dest, G_TYPE_DOUBLE);
			g_value_set_double(dest, src->floatValue);
			break;
		case TVALUE_STRING:
			g_value_init(dest, G_TYPE_STRING);
			g_value_set_string(dest, src->stringValue);
			break;

	}
}


void init_tvalue(TValue* tvalue)
{
	tvalue->valueType = TVALUE_NONE;

	tvalue->intValue = 0;
	tvalue->uintValue = 0;
	tvalue->longValue = 0;
	tvalue->ulongValue = 0;
	tvalue->floatValue = 0.0;
	tvalue->doubleValue = 0.0;

	tvalue->charValue = 0;
	tvalue->ucharValue = 0;
	tvalue->booleanValue = FALSE;
	tvalue->int64Value = 0;
	tvalue->uint64Value = 0;

	tvalue->flagsValue = 0;
	tvalue->enumValue = 0;

	tvalue->stringValue = NULL;

	tvalue->pointerValue = NULL;
	tvalue->boxedValue = NULL;
	tvalue->objectValue = NULL;
	tvalue->paramspecValue = NULL;

}

gboolean tvalue_holds_same_value(TValue* tvalue, GValue* exp_value)
{
	if(tvalue == NULL || exp_value == NULL || !G_IS_VALUE(exp_value)){
		return FALSE;
	}

	GType value_type = G_VALUE_TYPE(exp_value);

	if(G_TYPE_IS_FUNDAMENTAL(value_type)){
		switch(value_type){
			case G_TYPE_BOOLEAN:
				{
					gboolean bool_value = g_value_get_boolean(exp_value);
					return tvalue->booleanValue == bool_value;
				}
				break;

			case G_TYPE_CHAR:
				{
					gchar char_value = g_value_get_char(exp_value);
					return tvalue->charValue == char_value;
				}
				break;

			case G_TYPE_UCHAR:
				{
					guchar uchar_value = g_value_get_uchar(exp_value);
					return tvalue->ucharValue == uchar_value;
				}
				break;

			case G_TYPE_INT:
				{
					gint int_value = g_value_get_int(exp_value);
					return tvalue->intValue == int_value;
				}
				break;

			case G_TYPE_UINT:
				{
					guint uint_value = g_value_get_uint(exp_value);
					return tvalue->uintValue == uint_value;
				}
				break;

			case G_TYPE_LONG:
				{
					glong long_value = g_value_get_long(exp_value);
					return tvalue->longValue == long_value;
				}
				break;

			case G_TYPE_ULONG:
				{
					gulong ulong_value = g_value_get_ulong(exp_value);
					return tvalue->ulongValue == ulong_value;
				}
				break;

			case G_TYPE_INT64:
				{
					gint64 int64_value = g_value_get_int64(exp_value);
					return tvalue->int64Value == int64_value;
				}
				break;

			case G_TYPE_UINT64:
				{
					guint64 uint64_value = g_value_get_uint64(exp_value);
					return tvalue->uint64Value == uint64_value;
				}
				break;

			case G_TYPE_FLOAT:
				{
					gfloat float_value = g_value_get_float(exp_value);
					return tvalue->floatValue == float_value;
				}
				break;

			case G_TYPE_DOUBLE:
				{
					gdouble double_value = g_value_get_double(exp_value);
					return tvalue->doubleValue == double_value;
				}
				break;

			case G_TYPE_PARAM:
				{
					GParamSpec* pspec_value = g_value_get_param(exp_value);
					return tvalue->paramspecValue == pspec_value;
				}
				break;

			case G_TYPE_STRING:
				{
					const gchar* string_value = g_value_get_string(exp_value);
					if(tvalue->stringValue == string_value)
						return TRUE;
					else{
						if(tvalue->stringValue != NULL && string_value != NULL){
							return strncmp(tvalue->stringValue, string_value, strlen(string_value)) == 0;
						}else{
							return tvalue->stringValue == NULL && string_value == NULL;
						}
					}
				}
				break;

			case G_TYPE_POINTER:
				{
					gpointer pointer_value = g_value_get_pointer(exp_value);
					return tvalue->pointerValue == pointer_value;
				}
				break;

			case G_TYPE_OBJECT:
				{
					gpointer object_value = g_value_get_object(exp_value);
					return tvalue->objectValue == (GObject*)object_value;
				}
				break;

		}
	}else{
		if(G_TYPE_IS_ENUM(value_type)){
			gint enum_value = g_value_get_enum(exp_value);
			return tvalue->enumValue == enum_value;
		}else if(G_TYPE_IS_FLAGS(value_type)){
			gint flags_value = g_value_get_flags(exp_value);
			return tvalue->flagsValue == flags_value;
		}else if(G_TYPE_IS_BOXED(value_type)){
			if(value_type == G_TYPE_GSTRING){
				GString *sbs1 = (GString*)tvalue->boxedValue;
				GString *sbs2 = g_value_get_boxed(exp_value);
				return g_string_equal(sbs1, sbs2);
			}else{
				gpointer boxed_value = g_value_get_boxed(exp_value);
				return tvalue->boxedValue == boxed_value;
			}
		}
	}

	return FALSE;
}



// 1.if neither of pointers is NULL - return 0
// 2.if first pointer is NULL, and second is not - return 1
// 3.if second pointer is NULL, and first is not - return 2
// 4.if both pointers are NULL return 3
gint null_equal(gpointer p1, gpointer p2)
{
	if(p1 == NULL)
	{
		if(p2 == NULL)
			return 3;
		else
			return 2;
	}

	if(p2 == NULL)
		return 1;

	return 0;
}


gboolean gparamspec_equal(GParamSpec* pspec1, GParamSpec* pspec2)
{
	if(pspec1 == pspec2)
		return TRUE;

	gint nullEqValue = 0;
	if((nullEqValue = null_equal(pspec1, pspec2)) > 0)
		return nullEqValue == 3;

	if((nullEqValue = null_equal(pspec1->name, pspec2->name)) > 0)
		if(nullEqValue != 3)
			return FALSE;

	if(strncmp(pspec1->name, pspec2->name, strlen(pspec1->name)) != 0){
		gparamspec_comparison_print(pspec1, pspec2);
		return FALSE;
	}

	if(pspec1->flags != pspec2->flags){
		gparamspec_comparison_print(pspec1, pspec2);
		return FALSE;
	}
	if(pspec1->value_type != pspec2->value_type){
		gparamspec_comparison_print(pspec1, pspec2);
		return FALSE;
	}

	if(pspec1->owner_type != pspec2->owner_type){
		gparamspec_comparison_print(pspec1, pspec2);
		return FALSE;
	}

	return TRUE;
}

gboolean gvalue_equal(GValue* gvalue1, GValue* gvalue2)
{
	if(gvalue1 == gvalue2)
		return TRUE;

	gint null_eqc = 0;

	if((null_eqc = null_equal(gvalue1, gvalue2)) > 0){
		if(null_eqc == 3)
			return TRUE;
		else
			return FALSE;
	}

	if(!G_IS_VALUE(gvalue1)){
		return FALSE;
	}

	GType pType = 0;
	GType valueType = G_VALUE_TYPE(gvalue1);

	if(G_TYPE_IS_FUNDAMENTAL(valueType)){
		switch(valueType){
			case G_TYPE_BOOLEAN:
				pType = G_TYPE_PARAM_BOOLEAN;
				break;

			case G_TYPE_CHAR:
				pType = G_TYPE_PARAM_CHAR;
				break;

			case G_TYPE_UCHAR:
				pType = G_TYPE_PARAM_UCHAR;
				break;

			case G_TYPE_INT:
				pType = G_TYPE_PARAM_INT;
				break;

			case G_TYPE_UINT:
				pType = G_TYPE_PARAM_UINT;
				break;

			case G_TYPE_LONG:
				pType = G_TYPE_PARAM_LONG;
				break;

			case G_TYPE_ULONG:
				pType = G_TYPE_PARAM_ULONG;
				break;

			case G_TYPE_INT64:
				pType = G_TYPE_PARAM_INT64;
				break;

			case G_TYPE_UINT64:
				pType = G_TYPE_PARAM_UINT64;
				break;

			case G_TYPE_FLOAT:
				pType = G_TYPE_PARAM_FLOAT;
				break;

			case G_TYPE_DOUBLE:
				pType = G_TYPE_PARAM_DOUBLE;
				break;

			case G_TYPE_PARAM:
				pType = G_TYPE_PARAM_PARAM;
				break;

			case G_TYPE_STRING:
				pType = G_TYPE_PARAM_STRING;
				break;

			case G_TYPE_BOXED:
				pType = G_TYPE_PARAM_BOXED;
				break;

			case G_TYPE_POINTER:
				pType = G_TYPE_PARAM_POINTER;
				break;

			case G_TYPE_OBJECT:
				pType = G_TYPE_PARAM_OBJECT;
				break;

		}
	}else{
		if(G_TYPE_IS_ENUM(valueType)){
			pType = G_TYPE_PARAM_ENUM;
		}else if(G_TYPE_IS_FLAGS(valueType)){
			pType = G_TYPE_PARAM_FLAGS;
		}else if(G_TYPE_IS_BOXED(valueType)){
			pType = G_TYPE_PARAM_BOXED;
		}
	}

//	TRACE("param_type != G_TYPE_PARAM: %d", valueType != G_TYPE_PARAM)
//	TRACE("G_TYPE_IS_PARAM (pType): %d; %ld", G_TYPE_IS_PARAM (valueType), valueType)
//	TRACE("g_type_is_a =%d", g_type_is_a (valueType, G_TYPE_PARAM))
//	TRACE("FUNDAMENTAL=%d",G_TYPE_IS_FUNDAMENTAL(valueType))

//	TRACE("VALUE1 =%s; Type = %s", g_strdup_value_contents(gvalue1), G_VALUE_TYPE_NAME(gvalue1))
//	TRACE("VALUE2 =%s; Type = %s", g_strdup_value_contents(gvalue2), G_VALUE_TYPE_NAME(gvalue2))

	gboolean equalValues = FALSE;

	if(!G_TYPE_IS_BOXED(valueType)){
		GParamSpec* cmp_pspec = G_PARAM_SPEC(g_param_spec_internal(pType, "pspecCMP", "_p_spec_cmp__nick_", "", G_PARAM_READABLE));
		equalValues = g_param_values_cmp(cmp_pspec, gvalue1, gvalue2) == 0;
		g_param_spec_unref(cmp_pspec);

	}else{
		if(valueType == G_TYPE_GSTRING){
			GString *sbs1 = g_value_get_boxed(gvalue1);
			GString *sbs2 = g_value_get_boxed(gvalue2);
			equalValues = g_string_equal(sbs1, sbs2);
		}
	}

//	TRACE("equalValues = %d; sbs1->a = %s; sbs2->a = %s;", equalValues,  sbs1->str,  sbs2->str);

	return equalValues;
}

void gvalue_free(gpointer data){
	g_value_unset(data);
	g_free(data);
}

void gparamspec_comparison_print(GParamSpec* paramSpec, GParamSpec* expParamSpec)
{
	TRACE("\t GParamSpec pointer: %p; Expected value pointer: %p\n", (void*)paramSpec, (void*)expParamSpec)
	if(paramSpec != NULL && expParamSpec != NULL)
	{
		TRACE("\t GParamSpec.name: %s; Expected value: %s", paramSpec->name, expParamSpec->name)
		TRACE("\t GParamSpec.flags: %d; Expected value: %d", paramSpec->flags, expParamSpec->flags)
		TRACE("\t GParamSpec.value_type: %ld; Expected value: %ld", (long int)paramSpec->value_type, (long int)expParamSpec->value_type)
		TRACE("\t GParamSpec.owner_type: %ld; Expected value: %ld", (long int)paramSpec->owner_type, (long int)expParamSpec->owner_type)
	}
}
