#include "gttp-imap.h"



static void gttp_imap_base_init(gpointer g_class);
static void gttp_imap_base_finalize(gpointer g_class);
static void gttp_imap_class_init(gpointer g_class, gpointer class_data);

GType gttp_type_imap_t = 0;
GType gttp_type_imap = 0;

GType register_gttp_type_imap()
{
    if(gttp_type_imap == 0)
    {
        if(gttp_type_imap_t == 0)
        {
            gttp_type_imap = test_register_gttp_type_imap("GTTPIMap", 0, TRUE, TRUE, NULL);
        }else{
            gttp_type_imap = gttp_type_imap_t;
        }
    }
    return gttp_type_imap;
}



GTypeInfo imap_type_info;

GType test_register_gttp_type_imap(const gchar* type_name, GTypeFlags type_flags, gboolean base_init_enabled, gboolean class_init_enabled, const gpointer class_data)
{

//    if(gttp_type_imap_t != 0)
//        return gttp_type_imap_t;

    GBaseInitFunc base_init_func = base_init_enabled ? gttp_imap_base_init : NULL;
    GBaseFinalizeFunc base_finalize_func = gttp_imap_base_finalize;
    GClassInitFunc class_init_func =  class_init_enabled ? gttp_imap_class_init : NULL;


    imap_type_info.class_size = sizeof (GTTPIMapIface);
    imap_type_info.base_init = base_init_func;
    imap_type_info.base_finalize = base_finalize_func;
    imap_type_info.class_init = class_init_func;
    imap_type_info.class_finalize = NULL;
    imap_type_info.class_data = class_data;
    imap_type_info.instance_size = 0;
    imap_type_info.n_preallocs = 0;
    imap_type_info.instance_init = NULL;
    imap_type_info.value_table = NULL;


    const gchar* itype_name = type_name == NULL ? "GTTPIMap" : type_name;

    gttp_type_imap_t = g_type_register_static(G_TYPE_INTERFACE,
                                                     itype_name,
                                                     &imap_type_info,
                                                     type_flags);


    return gttp_type_imap_t;
}

static void gttp_imap_base_init(gpointer g_class)
{

//    TRACE("%s; type_name=%s; tid=%ld;gttp_type_imap=%d;g_class=%p", __func__, g_type_name_from_class(g_class), G_TYPE_FROM_INTERFACE(g_class), gttp_type_imap, g_class)

    if(IS_USED_CT(gttpimap_ct))
    {
        gttpimap_ct.base_init_call_count++;

        if(gttpimap_ct.base_init_call_count > gttpimap_ct.exp_base_init_call_count){
            TRACE("%s: GTypeInfo.base_init: the registered function for the interface was called more times than expected %d.",__func__, gttpimap_ct.exp_base_init_call_count);
            return;
        }

        if(IS_USED_CT(gttpfmap_ct))
        {
            //The implemented interface's base_init should be called first time *after* the implementing
            //  class' base_init()
            if(gttpfmap_ct.base_init_call_count == 0)
            {
                TRACE("%s: GTypeInfo.base_init: base_init() callback of the registered interface was called earlier than base_init() of the implementing class 'GTTPFMap'.",  __func__)
                gttpfmap_ct.iface_init_call_order_valid = FALSE;
                return;
            }
            gttpfmap_ct.iface_init_call_order_valid = TRUE;
        }

        if(IS_USED_CT(gttptreemap_ct))
        {
            //The implemented interface's base_init should be called third time *after* the implementing
            //  class' base_init() (GTTPTreeMap (impl GTTPIMap)-> GTTPFMap (impl GTTPIMap)
            if(gttptreemap_ct.base_init_call_count == 0 && gttpimap_ct.base_init_call_count == 3)
            {
                TRACE("%s: GTypeInfo.base_init: base_init() callback of the registered interface was called earlier than base_init() of the implementing class 'GTTPTreeMap'.",  __func__)
                gttptreemap_ct.iface_init_call_order_valid = FALSE;
                return;
            }
            gttptreemap_ct.iface_init_call_order_valid = TRUE;
        }



        if(g_class == NULL)
        {
            TRACE("%s: GTypeInfo.base_init: the first 'g_class' parameter passed in is NULL.", __func__)
            gttpimap_ct.base_init_params_valid = FALSE;
            return;
        }


        if(G_TYPE_FROM_INTERFACE(g_class) != gttp_type_imap_t)
        {
            TRACE("%s: GTypeInfo.base_init: the 'gpointer g_class' parameter passed is of invalid type, it must be of the same type as the interface itself.", __func__)
            gttpimap_ct.base_init_params_valid = FALSE;
            return;
        }

        if(gttpimap_ct.exp_base_init_call_count > 1 && gttpimap_ct.base_init_call_count > 1)
        {


            if(gttpimap_ct.class_init_p_exp_class == g_class)
            {
                TRACE("%s: GTypeInfo.base_init: the 'gpointer g_class' parameter passed is invalid, it must be vtable of the type of implementing class and not the orginal vtable.", __func__)
                gttpimap_ct.base_init_params_valid = FALSE;
                return;
            }

            if(IS_USED_CT(gttpfmap_ct))
            {
                //This will be used to verify, that implementing class receives correct interface vtable
                // in it's 'GInterfaceInfo.interface_init' callback.
                gttpfmap_ct.iface_base_init_call_p_g_class = g_class;
            }

            if(IS_USED_CT(gttptreemap_ct) && gttpimap_ct.base_init_call_count == 3)
            {
                //This will be used to verify, that implementing class receives correct interface vtable
                // in it's 'GInterfaceInfo.interface_init' callback.
                gttptreemap_ct.iface_base_init_call_p_g_class = g_class;
            }
        }

        if(!(((GTTPIMapIface*)g_class)->field_int == 0 && ((GTTPIMapIface*)g_class)->field_double == 0.0))
        {
            TRACE("%s: GTypeInfo.base_init: the fields of class structures were not zero initialized.", __func__)
            gttpimap_ct.struct_zero_filled = FALSE;
        }else{
            gttpimap_ct.struct_zero_filled = TRUE;
        }

        gttpimap_ct.base_init_params_valid = TRUE;

        //Setting original vtable on the first call of the 'base_init()', to be sure
        // that in each subsequent call the vtable passed in will be the copy, stored in the implementing class
        // and not the original one.
        if(gttpimap_ct.base_init_call_count == 1)
            gttpimap_ct.class_init_p_exp_class = g_class;
    }
}

static void gttp_imap_base_finalize(gpointer g_class)
{

}

static void gttp_imap_class_init(gpointer g_class, gpointer class_data)
{


//    TRACE("%s; type_name=%s; tid=%ld;gttp_type_imap=%d;g_class=%p", __func__, g_type_name_from_class(g_class), G_TYPE_FROM_INTERFACE(g_class), gttp_type_imap, g_class)
    if(IS_USED_CT(gttpimap_ct))
    {
        if(gttpimap_ct.base_init_call_count == 0)
        {
            TRACE("%s: GTypeInfo.class_init: class_init() callback is called earlier than base_init().",  __func__)
            gttpimap_ct.init_call_order_valid = FALSE;
            return;
        }

        if(gttpimap_ct.class_init_call_count > 0){
            TRACE("%s: GTypeInfo.class_init: the registered function for the interface was called multiple times.", __func__);
            return;
        }
        gttpimap_ct.class_init_call_count++;

        if(g_class == NULL)
        {
            TRACE("%s: GTypeInfo.class_init: the first 'g_class' parameter passed in is NULL.", __func__)
            gttpimap_ct.class_init_params_valid = FALSE;
            return;
        }

        if(g_class != gttpimap_ct.class_init_p_exp_class)
        {
            TRACE("%s: GTypeInfo.class_init: the 'gpointer g_class' parameter passed in is invalid.", __func__)
            gttpimap_ct.class_init_params_valid = FALSE;
            return;
        }

        if(class_data != gttpimap_ct.class_init_p_exp_class_data)
        {
            TRACE("%s: GTypeInfo.class_init: the 'gpointer class_data' parameter passed in does not have the expected value.", __func__)
            gttpimap_ct.class_init_params_valid = FALSE;
            return;
        }


        gttpimap_ct.init_call_order_valid = TRUE;
        gttpimap_ct.class_init_params_valid = TRUE;
    }
}
